#! /usr/bin/python

from xml.dom.ext.reader import Sax2
from xml.dom.ext import Print
import os
import stat
import fnmatch
import string
import re
import sys
import getopt
from urlparse import urlparse

def help():
    print '''zap2al [option1 [,option2...]] [pattern1 [pattern2...]]
converts a zaplet files to a part of tree the junkbuster part of Al may use
the converted files are found in dirname(patternn) recursively and are treated
if their name matches basename(patternn)
if no pattern is specified, zaplets are read on standard input
available options :
  -d, --document
    output will be a full XML document
  -h, --help
    this help screen
  -p, --popups-forbidden
    ignore files containing block-popups markups
  -s, --strict
    ignore files containing unknown markups
  -v, --verbose
    display more information (so you know it did not hang :=)'''

def pattern2list(path,pattern):
    try:
        if stat.S_ISDIR(os.stat(path)[stat.ST_MODE]):
            l=[]
            for foo in os.listdir(path):
                l=l+pattern2list(path+'/'+foo,pattern)
            return l
        else:
            if fnmatch.fnmatchcase(os.path.basename(path),pattern):
                if os.access(path,os.R_OK):
                    return [path]
                else:
                    sys.stdout.write(path+' : '+'not readable\n')
                    return []
            return []
    except OSError,(code,msg):
        sys.stdout.write(path+' : '+msg+'\n')
        return []

class ZapletError(Exception):
    pass

def next(element):
    while 1:
         if element.nodeType==element.ELEMENT_NODE:
             return element
         element=element.nextSibling

def specificity(url):
    # beurk : a very very unprecise way of determining the specificity of an
    # url matching regexp
    return string.count(url,':')+string.count(url,'\.')+string.count(url,'\?')+\
           string.count(url,'#')+string.count(url,'=')

def zaplet2(zaplets):
    list3=[]
    list4=[]
    for filename,zf in zaplets:
        try:
            if verbose:
                print 'converting '+filename
            try:
                z=Sax2.FromXml(zf).documentElement
            except KeyboardInterrupt:
                raise KeyboardInterrupt
            except:
                raise ZapletError,'not a valid xml file'
            version=None
            rules={}
            host=None
            list2=[]
            known_markup=0
            filters=[]
            if z.nodeName!='zaplet':
                raise ZapletError,'first element is not <zaplet>'
            try:
                z=z.firstChild
                z=next(z)
            except:
                raise ZapletError,'<zaplet> has no element children'
            try:
                doc=z.ownerDocument
                while 1:
                    if z.nodeName=='version':
                        version=string.strip(z.firstChild.data)
                        if version!='0.9':
                            raise ZapletError,'only recognizes zaplet version 0.9'
                        z=next(z.nextSibling)
                        continue
                    if z.nodeName=='host':
                        host=string.strip(z.firstChild.data)
                        z=next(z.nextSibling)
                        continue
                    processing_type=z.nodeName
                    if processing_type=='block_url':
                        rule='trash'
                        known_markup=1
                    if processing_type=='allow_url':
                        rule='allow'
                        known_markup=1
                    if known_markup:
                        foo=doc.createElement(rule)
                        list2.append(foo)
                        rules[foo]=[]
                        while z.nodeName==processing_type:
                            bar=doc.createElement('rule')
                            ztype=z.getAttributeNS('','type')
                            if ztype=='regex':
                                if (not z.firstChild) or z.firstChild.nodeType!=z.TEXT_NODE:
                                    raise ZapletError,'<'+processing_type+'> should have exactly one text node as child'
                                bar.setAttribute('url','.*'+z.firstChild.data+'.*')
                                spec=specificity(z.firstChild.data)
                            elif ztype=='literal':
                                if (not z.firstChild) or z.firstChild.nodeType!=z.TEXT_NODE:
                                    raise ZapletError,'<'+processing_type+'> should have exactly one text node as child'
                                zarma=re.escape(z.firstChild.data)
                                bar.setAttribute('url','.*'+zarma+'.*')
                                spec=specificity(zarma)
                            elif ztype!='everything':
                                raise ZapletError,'rule\'s type should be "literal", "regex", or "everything"'
                            else:
                                spec=0
                            rules[foo].append((spec,bar))
                            z=next(z.nextSibling)
                        known_markup=0
                        continue
                    if processing_type=='filter':
                        filter=doc.createElement('filter')
                        foo=next(z.firstChild)
                        try:
                            while 1:
                                if foo.nodeName=='filter_match_url':
                                    bar=foo.getAttributeNS('','type')
                                    try:
                                        if bar=='literal':
                                            u=doc.createElement('url_match')
                                            filter.appendChild(u)
                                            m='.*'+re.escape(foo.firstChild.data)+'.*'
                                            u.appendChild(doc.createTextNode(m))
                                        elif bar=='regex':
                                            u=doc.createElement('url_match')
                                            filter.appendChild(u)
                                            m=foo.firstChild.data
                                            u.appendChild(doc.createTextNode(m))
                                        elif bar!='everything':
                                            raise ZapletError,'filter_match_url\'s type should be set to "literal", "regex", or "everything"'
                                    except AttributeError:
                                        raise ZapletError,'filter_match_url node should have one text node as child'
                                elif foo.nodeName=='filter_match_text':
                                    t=doc.createElement('text_match')
                                    filter.appendChild(t)
                                    bar=foo.getAttributeNS('','type')
                                    try:
                                        if bar=='literal':
                                            m=re.escape(foo.firstChild.data)
                                            t.appendChild(doc.createTextNode(m))
                                        elif bar=='regex':
                                            m=foo.firstChild.data
                                            t.appendChild(doc.createTextNode(m))
                                        else:
                                            raise ZapletErrot,'filter_match_text\'s type should be set to "literal" or "regex"'
                                    except AttributeError:
                                        raise ZapletError,'filter_match_text node should have one text node as child'
                                elif foo.nodeName=='filter_replace_text':
                                    t=doc.createElement('replace_by')
                                    filter.appendChild(t)
                                    try:
                                        bar=foo.firstChild.data
                                        t.appendChild(doc.createTextNode(bar))
                                    except AttributeError:
                                        raise ZapletError,'filter_replace_text node should have one text node as child'
                                foo=next(foo.nextSibling)
                        except AttributeError:
                            filters.append(filter)
                            z=next(z.nextSibling)
                            continue
                    if popups_forbidden and processing_type=='block_popups':
                        raise ZapletError,'block_popup markup'
                    while z.nodeName==processing_type:
                        if strict:
                            raise ZapletError,'unknown markup '+processing_type
                        else:
                            sys.stderr.write('ignoring unknown markup '+processing_type+' and its children\n')
                        z=next(z.nextSibling)
                    z=next(z.nextSibling)
            except AttributeError:
                if not version:
                    raise ZapletError,'zaplet version number unspecified'
                if not host:
                    raise ZapletError,'host unspecified'
                one_trash=0
                for foo in list2:
                    one_trash=one_trash|(foo.nodeName=='trash')
                    if len(rules[foo])<2:
                        foo.appendChild(rules[foo][0][1])
                        if host not in ['default-numeric','default']:
                            rules[foo][0][1].setAttribute('host','.*'+re.escape(host)+'.*')
                    else:
                        if host not in ['default-numeric','default']: 
                            andn=doc.createElement('and')
                            foo.appendChild(andn)
                            bar=doc.createElement('rule')
                            bar.setAttribute('host','.*'+re.escape(host)+'.*')
                            andn.appendChild(bar)
                            orn=doc.createElement('or')
                            andn.appendChild(orn)
                            rules[foo].sort()
                            rules[foo].reverse()
                            for spec,bar in rules[foo]:
                                orn.appendChild(bar)
                        else:
                            rules[foo].sort()
                            for spec,bar in rules[foo]:
                                foo.appendChild(bar)
                if one_trash:
                    t=specificity(re.escape(host))
                    list3=list3+map(lambda x,y=t:(y,x),list2)
                for filter in filters:
                    if host not in ['default-numeric','default']:
                        filter.setAttributeNS('','host','.*'+re.escape(host)+'.*')
                list4=list4+filters
        except ZapletError,msg:
            sys.stderr.write('In '+filename+' : '+msg[0]+'. Ignoring this file\n')
        except foo,tuple:
            sys.stderr.write('Error when trying to convert '+zf+'\n')
    list3.sort()
    list3.reverse()
    return map(lambda x:x[1],list3)+list4

document=0
popups_forbidden=0
strict=0
verbose=0
try:
    opt,remaining=getopt.getopt(sys.argv[1:],'dhpsv',
                                ['document','help','popus-forbidden','strict',
                                 'verbose'])
    for foo,bar in opt:
        if foo in ['-d','--document']:
            document=1
        if foo in ['-h','--help']:
            help()
            sys.exit(0)
        if foo in ['-p','--popups-forbidden']:
            popups_forbidden=1
        if foo in ['-s','--strict']:
            strict=1
        if foo in ['-v','--verbose']:
            verbose=1
except getopt.error,x:
    sys.stderr.write(x)
    sys.exit(2)
if document:
    print '<?xml version="1.0"?>'
    print '<spam-policy>'
    print '<policy>'
if not remaining:
    XB='[ \t]*'
    XBX='['+string.whitespace+']*'
    bar=[]
    i=1
    for tree in re.split(XBX+'<'+XB+'/'+XB+'zaplet'+XB+'>'+XBX,
                         string.strip(sys.stdin.read())):
        i=i+1
        if tree:
            bar.append(('zaplet '+str(i),tree+'</zaplet>'))
else:
    for path in remaining:
        dir=os.path.dirname(path)
        bas=os.path.basename(path)
        if not dir:
            dir='.'
        if not bas:
            bas='*.zap'
        bar=[]
        for foo in pattern2list(dir,bas):
            try:
                z=open(foo)
                bar.append((dir+'/'+bas,z.read()))
                z.close()
            except:
                sys.stderr.write('an error occured while reading '+dir+'/'+bas)
for foo in zaplet2(bar):
    Print(foo)
    print
if document:
    print '</policy>'
    print '</spam-policy>'
