# Copyright (c) 2000 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# Some parts of code have been taken from package PyXML-0.6.5 
# Copyright (c) 2000 Fourthought Inc, USA 
# License Agreement and CNRI's notice of copyright, i.e., "Copyright (c)
# Copyright (c) 1991 - 1995, Stichting Mathematisch Centrum Amsterdam,
# Copyright (c) 1998 by Secret Labs AB.   

from narval.lib import *

MOD_XML='''<?xml version="1.0" encoding="ISO-8859-1"?><module name='Bookmarks'>'''

XBEL_NS = ''
CHKS_NS = 'http://www.logilab.org/narval'

#
# Extractor class
#
import copy, re, xmllib
from xml.sax import saxexts

bookmark = load_extension_module('xbel','bookmark')
ns_parse = load_extension_module('xbel','ns_parse')
lynx_parse = load_extension_module('xbel','lynx_parse') 
# ns_parse=load_extension_module('xbel','msie_parse')

# Convert the Bookmarks file into an xbel file
def parse_bmf_f(args):
    doc = args.ownerDocument
    output=doc.createElementNS(XBEL_NS,'outputs')
    child=args.firstChild
    type=child.getAttributeNS(XBEL_NS,'type')

    #check the browser type (netscape, lynx, opera ...)
    if type == 'netscape':
        ns_handler=ns_parse.NetscapeHandler()
        p=saxexts.SGMLParserFactory.make_parser()
        p.setDocumentHandler(ns_handler)
        data=select_one(args,'bookmark-file/text()').data 
        input_file = open(expand_vars(data), 'r')
        strhtml=input_file.read()
        strhtml=latin1_to_utf8(strhtml)
        p.feed(strhtml)
        bms = ns_handler.bms
        domTree=bms.dump_xbel(doc)
        output.appendChild(domTree)
    elif type == 'lynx':
        print 'Warning : Beta version for lynx'
        bms = bookmark.Bookmarks()
        # Determine the owner on Unix platforms
        import os, pwd, glob, sys
        uid = os.getuid()
        t = pwd.getpwuid( uid )
        bms.owner = t[4]
        domTree=doc.createElementNS(XBEL_NS,'xbel')
        glob_pattern = os.path.join(sys.argv[1], '*.html')
        file_list = glob.glob( glob_pattern )
        for file in file_list:
            input = open(file)
            lynx_parse.parse_lynx_file(bms, input)
            child=bms.dump_xbel(doc)
            domTree.appendChild(child)
        
        output.appendChild(domTree)
    elif type == 'raw':
        url=select_one(args,'bookmark-file/text()').data
        try:
            f = urllib.urlopen(normalize_url(url)[0])
            xbel = doc.createElementNS('', 'xbel')
            import re
            url_list = re.split('[\n]', data)
            for url in url_list:
                if url == '':
                    continue
                bk = doc.createElementNS('', 'bookmark')
                title = doc.createElementNS('', 'title')
                title.appendChild(doc.createTextNode(url))
                bk.appendChild(title)
                bk.setAttributeNS('', 'href', url)
                xbel.appendChild(bk)
            output.appendChild(xbel)
        except IOError:
            Err = doc.createElementNS('','error')
            Err.setAttributeNS('','type','No such url '+url)
            outputsNode.appendChild(Err)
            return outputsNode
    else:
        print 'browser type unknown : ', type
    return output

MOD_XML=MOD_XML+'''
<action name='parse_bmf' func='parse_bmf_f'>
<description lang="en">parse bookmark file from several formats and transform it
in an xbel elment</description>
  <input><match>bookmark-file[@type]/text()</match>
<description lang="en">url of the original bookmark file</description></input>
  <output><match>xbel</match>
<description lang="en">the xbel bookmarks tree</description></output>
</action>
''' 

def update_bmf_f(args):
    """
    Update the $NARVAL_HOME/data/bookmarks.xbel file 
    """

    doc = args.ownerDocument
    output=doc.createElementNS(XBEL_NS,'outputs')
    
    data=select_one(args,'bookmark-file/text()').data
    sourceXbel=select_one(args,'xbel')
    
    try:
        refXbel=read_domtree_from_uri(data)
    except:
        write_domtree_to_file(sourceXbel,data)
        output.appendChild(sourceXbel)
        return output
    
    sourceElementList=sourceXbel.childNodes
    
    # Browse the "new" xbel file
    for sourceElement in sourceElementList:
        nodeName=sourceElement.nodeName
        
        # Call the appropriate function
        if nodeName == 'bookmark':
            update_bookmark_element(sourceElement,refXbel.documentElement)
        elif nodeName == 'folder':
            update_folder_element(sourceElement,refXbel.documentElement)

    
    write_domtree_to_file(sourceXbel,data)
    output.appendChild(sourceXbel)
    return output
    

MOD_XML=MOD_XML+'''
<action name='update_bmf' func='update_bmf_f'>
<description lang="en">update checksum for the xbel bookmarks pages and update
the xbel bookmarks file. Return the old xbel tree</description>
<input><match>bookmark-file[@type="xbel"]/text()</match>
<description lang="en">the xbel bookmarks to update</description></input>
<input><match>xbel</match>
<description lang="en">the new xbel bookmarks to update</description></input>
<output><match>xbel</match>
<description lang="en">the original xbel tree</description></output>
</action>
'''

MOD_XML=MOD_XML+'</module>'


# Update a bookmark element
def update_bookmark_element(source,refXbel):
    # retrieve the url 
    adr=source.getAttributeNS(XBEL_NS,'href')
    # try to find it in the old file
    refBms=select_many(refXbel,'//bookmark[@href="%s"]'%adr)
    # if more than one was found, then take (arbitrary) the first one
    if refBms:
        chks=refBms[0].getAttributeNS(CHKS_NS,'checksum')
        source.setAttributeNS(CHKS_NS,'n:checksum',chks)
    #else, nothing to be done

#Update a folder element
def update_folder_element(source,refXbel):
    sourceElementList = source.childNodes
    # Browse all the folder childs
    for sourceElement in sourceElementList:
        nodeName=sourceElement.nodeName
        # Call the appropriate function
        if nodeName == 'bookmark':
            update_bookmark_element(sourceElement,refXbel)
        elif nodeName == 'folder':
            update_folder_element(sourceElement,refXbel)


    

if __name__ == "__main__" :
    from xml.dom.ext.reader import Sax2
    print MOD_XML
    doc=Sax2.FromXml(MOD_XML)
    print doc.documentElement
